<?php
require_once(PFAD_ROOT.PFAD_INCLUDES_MODULES.'ServerPaymentMethod.class.php');
require_once(dirname(__FILE__).'/../mcpay/classes/class.mcpay.php');

class micropayment_base extends ServerPaymentMethod
{
  var $paymethod = '';
  /** @var    mcpay $McPay */
  var $McPay;
  //var $debug       = FALSE;
  var $localeTexts = array();

  /**
   * init
   *
   * @return \micropayment_base|void
   */
  function init()
  {
    parent::init();

    $this->name      = 'Micropayment';
    $this->caption   = 'Micropayment';
    //$this->paymethod = 'ccard_web';

    $sql        = "SELECT * FROM tzahlungsart WHERE cModulId = '{$this->cModulId}'";
    $this->info = $GLOBALS["DB"]->executeQuery($sql, 1);

    // load texts
    $this->loadTexts();
  }

  /**
   * micropayment_base constructor.
   *
   * @param     $moduleID
   * @param int $nAgainCheckout
   */
  function __construct($moduleID = NULL, $nAgainCheckout = 0)
  {
    $this->pageURL = 'http://'.$_SERVER['HTTP_HOST'].'/';
    if ($this->isHTTPS()) $this->pageURL = 'https://'.$_SERVER['HTTP_HOST'].'/';

    $oPlugin = Plugin::getPluginById('mcpay_mircopayment');
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$oPlugin->oPluginEinstellungAssoc_arr', print_r($oPlugin->oPluginEinstellungAssoc_arr,1));
    $this->McPay = new mcpay((bool)$oPlugin->oPluginEinstellungAssoc_arr['testmode']);
    $this->debug        = (bool)$oPlugin->oPluginEinstellungAssoc_arr['micropayment_debug'];
    $this->McPay->debug = (bool)$oPlugin->oPluginEinstellungAssoc_arr['micropayment_debug'];

    if (!empty($moduleID)){
      parent::__construct($moduleID, $nAgainCheckout);
    } else {
      $this->loadSettings();
      $this->init($nAgainCheckout);
    }
  }

  /**
   * getPluginVersion
   *
   * @return mixed
   */
  function getPluginVersion()
  {
    global $db;
    $sql = 'SELECT * FROM `tplugin` WHERE `cPluginID` = "mapa" ';
    $res = $GLOBALS["DB"]->executeQuery($sql, 1);
    return $res->nXMLVersion;
  }

  /**
   * isHTTPS
   *
   * @return bool
   */
  function isHTTPS()
  {
    if (strpos($_SERVER['HTTP_HOST'], '.local') === FALSE) {
      if (!isset($_SERVER['HTTPS']) || (strtolower($_SERVER['HTTPS']) != 'on' && $_SERVER['HTTPS'] != '1')) {
        return FALSE;
      }
    }
    return TRUE;
  }

  /**
   * setStatus
   *
   * sets status to order and marks as not synced again
   *
   * @param $status
   * @param $orderId
   * @param $comment
   *
   * @return bool
   */
  function setStatus($status, $orderId, $comment = NULL)
  {
    if ($status != '') {
      $sql = '
        UPDATE `tbestellung` 
        SET `dBezahltDatum` = NOW(),
          `cStatus` = "'.$status.'",
          `cAbgeholt` = "N"
          ';
      if (!empty($comment)) {
        $sql .= ', `cKommentar` = "'.addslashes(utf8_decode($comment)).'" ';
      }
      $sql.= 'WHERE `cBestellNr` = "'.addslashes($orderId).'"';
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->sql: '.print_r($sql, 1));
      $GLOBALS["DB"]->executeQuery($sql, 3);
    }
    return TRUE;
  }

  /**
   * getUserDataFromOrder
   *
   * @param $order
   * @param $config
   *
   * @return array
   */
  function getUserDataFromOrder($order, $config)
  {
    $this->McPay->setConfig($config);

    $custID  = $_SESSION['Kunde']->kKunde;
    $orderId = $order->cBestellNr;
    if (empty($orderId)) $orderId = baueBestellnummer();

    if (empty($_SESSION['Kunde'])) { // guest order
      $custID = 'guest-'.$orderId;
    } else {
      $orderId .= '-'.$custID;
    }
    $custID = $this->McPay->unifyID($custID);

    $currency = $_SESSION['Waehrung']->cISO;
    $amount   = $order->fGesamtsummeKundenwaehrung; // In customer currency
    if (empty($amount)) $amount = $_SESSION["Warenkorb"]->gibGesamtsummeWaren(1);
    $amount   = round($amount * 100);

    $user     = $_SESSION['Kunde'];
    $oPlugin  = $this->getPluginObj($order);
    $chksumData = array(
      'amount'        => $amount,
      'currency'      => $currency,
      'iso'           => $user->cLand,
      'host'          => $_SERVER['HTTP_HOST'],
      'project'       => $config['project'],
      'shoptype'      => 'jtl',
      'shopversion'   => (string)(JTL_VERSION/100).'.'.JTL_MINOR_VERSION,
      'plugintype'    => 'API',
      'pluginversion' => implode('.', str_split((string)$oPlugin->nVersion)),
    );
    $userData = array(
      'company'    => $user->cFirma,
      'firstName'  => $user->cVorname,
      'lastName'   => $user->cNachname,
      'salutation' => $user->cAnrede == 'm' ? 'MR' : 'MRS',
      'street'     => $user->cStrasse.' '.$user->cHausnummer,
      'zip'        => $user->cPLZ,
      'city'       => $user->cOrt,
      'country'    => $user->cLand,
      'email'      => $user->cMail,
      'ip'         => '', //$_SESSION['oBesucher']->cIP,
      'id'         => $custID,
      'order_id'   => $orderId,
      'lang'       => $config['lang'],
      'amount'     => $amount,
      'currency'   => $currency,
      'chksum'     => $chksumData, // request security
    );
    if (empty($userData['ip'])) $userData['ip'] = $_SERVER['REMOTE_ADDR']; // Falls IP Leer, dann aus dem Server holen
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$userData', print_r($userData,1));
    return $userData;
  }

  /**
   * handleNotification
   *
   * @param \Bestellung $order
   * @param string      $paymentHash
   * @param array       $args
   * @param bool        $returnURL
   *
   * @return string|void
   */
  function handleNotification($order, $paymentHash, $args, $returnURL = FALSE)
  {
    $crlf = "\r\n";
    //$args = func_get_args();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$args', print_r($args, 1).print_r($order,1));
    if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->args: '.print_r($args, 1));

    $orderID = $order->cBestellNr;
    if (empty($orderID) && !empty($args['orderid'])) {
      $orderID = $args['orderid'];
      if (strpos($orderID, '-')) {
        $p       = explode('-', $orderID);
        $orderID = $p[0];
      }
    }

    $trxID = $args['transactionId'];
    if (empty($trxID) || $trxID == '__transactionId__'){
      $trxID = $args['auth'];
    }

    $incomingPayment           = new stdClass();
    $incomingPayment->fBetrag  = $order->fGesamtsummeKundenwaehrung;
    $incomingPayment->cISO     = $order->Waehrung->cISO;
    $incomingPayment->cHinweis = $trxID;

    $return = 'status=ok'.$crlf;

    switch($args['function']){
      case 'custinfo':
        break;
      case 'billing':
        $url   = $this->getReturnURL($order);

        $this->addIncomingPayment($order, $incomingPayment);
        $this->setOrderStatusToPaid($order);
        $this->sendConfirmationMail($order);
        $this->updateNotificationID($order->kBestellung, $paymentHash);

        $return = 'status=ok'.$crlf // ok  | error
          .'url='.$url.$crlf
          .'target=_top'.$crlf
          .'forward=1'.$crlf
          .'message=OK'.$crlf;
        break;

      case 'storno':
        $incomingPayment->fBetrag  = ($args['amount'] / 100) * -1;
        $incomingPayment->cISO     = $args['currency'];

        $this->addIncomingPayment($order, $incomingPayment);
        $this->setStatus(BESTELLUNG_STATUS_STORNO, $orderID);
        break;

      case 'backpay':
        $incomingPayment->fBetrag  = ($args['amount'] / 100);
        $incomingPayment->cISO     = $args['currency'];

        $this->addIncomingPayment($order, $incomingPayment);
        $this->setStatus(BESTELLUNG_STATUS_IN_BEARBEITUNG, $orderID);
        break;

      case 'refund':
        $incomingPayment->fBetrag  = ($args['amount'] / 100) * -1;
        $incomingPayment->cISO     = $args['currency'];

        $this->addIncomingPayment($order, $incomingPayment);
        $this->setStatus(BESTELLUNG_STATUS_IN_BEARBEITUNG, $orderID);
        break;

      case 'quit':
        $incomingPayment->fBetrag  = $order->fGesamtsummeKundenwaehrung * -1;
        $incomingPayment->cISO     = $order->Waehrung->cISO;

        $this->addIncomingPayment($order, $incomingPayment);
        $this->setStatus(BESTELLUNG_STATUS_STORNO, $orderID);
        break;

      case 'init': // prepay
        /*
        $incomingPayment->fBetrag  = ($args['amount'] / 100);
        $incomingPayment->cISO     = $args['currency'];

        $this->addIncomingPayment($order, $incomingPayment);
        */
        $this->setStatus(BESTELLUNG_STATUS_IN_BEARBEITUNG, $orderID);
        break;
    }
    if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->return: '.print_r($return, 1));
    if ($returnURL) return $return;
    echo $return;
  }

  /**
   * preparePaymentProcess
   *
   * @param \Bestellung $order
   */
  function preparePaymentProcess($order)
  {
    $config = $this->getPluginConf($order);
    //echo '<pre>'.print_r($config,1).'</pre>';
    //echo '<pre>'.print_r($order,1).'</pre>';
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$order', print_r($order,1));
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$this', print_r($this,1));
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->Warenkorb', print_r($_SESSION['Warenkorb'],1));

    global $Einstellungen;
    if ($Einstellungen['kaufabwicklung']['bestellabschluss_abschlussseite'] !== 'A') {
      if ($this->info->nWaehrendBestellung == 0){
        die(
          'Bitte Abschlussseite nach externer Bezahlung auf Abschlussseite stellen. <br>'.
          '(Storefront -> Kaufabwicklung -> Warenkorb/Kaufabwicklung -> Bestellabschluss -> Abschlussseite nach externer Bezahlung)<br><br>'.
          'oder<br><br> Zahlung vor Bestellabschluss im Zahlverfahren auf Ja stellen.<br>'.
          '(Storefront -> Zahlarten -> Übersicht -> '.$this->info->cName.')'
        );
      }
    }

    if ($order) {
      $userData = $this->getUserDataFromOrder($order, $config);

      //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$userData', print_r($userData,1));
      //echo '<pre>'.print_r($userData, 1).'</pre>';

      $hash = $this->generateHash($order);
      if ($order->cId != '') $hash = $order->cId;

      $notifyURL                         = $this->getNotificationURL($hash);
      $config['returnURL']               = $notifyURL.'&mcpaySessID=';
      $config['returnURLNO3D']           = $notifyURL.'&mcpaySessID=';
      $key                               = ($this->duringCheckout) ? 'sh' : 'ph';
      $config['extraParams'][$key]       = $hash;
      $config['extraParams']['sessName'] = session_name();
      $config['extraParams']['sessId']   = session_id();

      $url = $this->McPay->getPaymentWindowParams($userData, $config, $this->pm);
      //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$url', print_r($url, 1));

      if (!empty($url)) {
        header('Location: '.$url);
        exit();
      }
    }
  }

  /**
   * handleAdditional
   * validate additional page post data
   *
   * @param array $aPost_arr
   *
   * @return bool
   */
  function handleAdditional($req)
  {
    return TRUE; // hide zusatzschritt
  }

  /**
   * redirectOnPaymentSuccess
   *
   * @return bool
   */
  function redirectOnPaymentSuccess()
  {
    //$args = func_get_args();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$args', print_r($args,1));
    return FALSE;
  }

  /**
   * redirectOnCancel
   *
   * @return bool
   */
  function redirectOnCancel()
  {
    //$args = func_get_args();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$args', print_r($args,1));
    return FALSE;
  }

  /**
   * getReturnURL
   *
   * @param \Bestellung $order
   *
   * @return string
   */
  function getReturnURL($order)
  {
    //$args = func_get_args();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$args', print_r($args, 1));
    return parent::getReturnURL($order);
  }

  /**
   * finalizeOrder
   *
   * @param \Bestellung $order
   * @param string      $hash
   * @param array       $args
   *
   * @return bool|true
   */
  function finalizeOrder($order, $hash, $args)
  {
    //$args = func_get_args();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$args', print_r($args, 1).print_r($_POST, 1));

    return TRUE;
  }

  /**
   * getPluginObj
   *
   * @param $order
   *
   * @return bool|\Plugin
   */
  function getPluginObj($order)
  {
    $myZV = $_SESSION['Zahlungsart']->cModulId;
    if (empty($myZV)) $myZV = $order->Zahlungsart->cModulId;

    $this->moduleID = $myZV;
    $this->init(0);

    $kPlugin = gibkPluginAuscModulId($myZV);
    if ($kPlugin > 0) {
      $oPlugin = new Plugin($kPlugin);
    } else {
      return FALSE;
    }

    return $oPlugin;
  }

  /**
   * getPluginConf
   *
   * @param      $order
   * @param null $oPlugin
   *
   * @return array|bool
   */
  function getPluginConf($order, $oPlugin = NULL)
  {
    if (!is_object($oPlugin)) {
      $oPlugin = $this->getPluginObj($order);
    }
    if (!is_object($oPlugin)) return FALSE;

    //echo '<pre>'.print_r($oPlugin,1).'</pre>';
    //echo '<pre>Order:'.print_r($order,1).'</pre>';
    //echo '<pre>'.print_r($_SESSION['Zahlungsart'],1).'</pre>';
    //$oPlugin->oPluginZahlungsmethodeAssoc_arr;
    $conf                   = $oPlugin->oPluginEinstellungAssoc_arr;
    $conf['plugin_url']     = $oPlugin->cFrontendPfadURLSSL;
    $conf['plugin_url']     = str_replace('frontend', 'mcpay', $conf['plugin_url']);
    $conf['plugin_url']     = str_replace('http://', 'https://', $conf['plugin_url']);
    $conf['plugin_path']    = $oPlugin->cFrontendPfad;
    $conf['plugin_path']    = str_replace('frontend', 'mcpay', $conf['plugin_path']);
    $conf['plugin_shopurl'] = URL_SHOP;
    $conf['plugin_shopurl']     = str_replace('http://', 'https://', $conf['plugin_shopurl']);
    if (empty($conf['payformid'])) {
      $conf['payformid'] = 'form_payment_extra'; // jtl default
    }
    //echo '<pre>'.print_r($conf, 1).'</pre>';
    //echo '<pre>'.print_r($oPlugin,1).'</pre>';
    $lang = $_SESSION['cISOSprache'] == 'ger' ? 'DE' : 'EN';

    $config = array(
      'project'         => $conf['project_id'],
      'accessKey'       => $conf['accesskey'],
      'testMode'        => $conf['testmode'],
      'suffix'          => $conf['suffix'],
      'theme'           => $conf['theme'],
      'gfx'             => $conf['gfx'],
      'bgcolor'         => $conf['bgcolor'],
      'bggfx'           => $conf['bggfx'],
      'paytext'         => $conf['paytext'],
      'plugin_urljs'    => $conf['plugin_shopurl'].'/mcpayGetScript?sn=',
      'plugin_urlcss'   => $conf['plugin_shopurl'].'/mcpayGetStyle?sn=',
      'plugin_urlshop'  => $conf['plugin_shopurl'],
      'plugin_path'     => $conf['plugin_path'].'',
      'plugin_pathview' => $conf['plugin_path'].'view/',
      'plugin_pathjs'   => $conf['plugin_path'].'js/',
      'plugin_pathcss'  => $conf['plugin_path'].'css/',
      'payformid'       => $conf['payformid'],
      'debug'           => $conf['micropayment_debug'],
      'customPrefix'    => $conf['customPrefix'],
      'lang'            => $lang,
      'expiredays'      => $conf['expiredays'],
      'start3D'         => $conf['start3D'],
      'start3Dreuse'    => $conf['start3Dreuse'],
    );
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$config', print_r($config,1));
    $this->McPay->setConfig($config);
    return $config;
  }

  /**
   * loadTexts
   *
   */
  function loadTexts()
  {
    $kPlugin = gibkPluginAuscModulId($this->cModulId);
    if ($kPlugin > 0) {
      $oPlugin = new Plugin($kPlugin);
    }
    $texts = $oPlugin->oPluginSprachvariableAssoc_arr;
    //echo '<pre>'.print_r($texts, 1).'</pre>';
    $this->setTexts($texts);
  }

  /**
   * setTexts
   *
   * @param $texts
   */
  function setTexts($texts)
  {
    $this->localeTexts = $texts;
  }

  /**
   * getText
   *
   * @param $textName
   * @param $default
   *
   * @return mixed
   */
  function getText($textName, $default)
  {
    if (empty($this->localeTexts[$textName])) {
      return $default;
    }
    return $this->localeTexts[$textName];
  }

  /**
   * getStyleAction
   *
   * @param $oPlugin
   *
   * @return bool
   */
  function getStyleAction($oPlugin)
  {
    header('Content-Type: text/css');
    $req = $_REQUEST;
    if (empty($req['sn'])) {
      if ($this->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->Req empty: '.print_r($req['sn'], 1));
      echo '403';
      return FALSE;
    }

    $config = $this->getPluginConf('', $oPlugin);
    //echo '<pre>'.print_r($config, 1).'</pre>';
    $filename = $config['plugin_pathcss'].$req['sn'].'';
    if (!file_exists($filename)) {
      if ($this->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->File not found: '.print_r($filename, 1));
      echo '404';
      return FALSE;
    }

    $content = file_get_contents($filename);

    if (strpos($filename, 'mcpay_formstyle_') !== FALSE) {
      $repl    = array(
        "url('../fonts/" => "url('".$config['plugin_urlfont'],
        'url("../img/icon_' => 'url("'.$config['plugin_urlcss'].'../img/icon_',
      );
      $content = strtr($content, $repl);
    }
    echo $content;
  }

  /**
   * getScriptAction
   *
   * @param $oPlugin
   *
   * @return bool
   */
  public function getScriptAction($oPlugin)
  {
    header('Content-Type: application/javascript');
    $req = $_REQUEST;
    if (empty($req['sn'])) {
      if ($this->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->Req empty: '.print_r($req['sn'], 1));
      echo '403';
      return FALSE;
    }

    $config = $this->getPluginConf('', $oPlugin);
    //echo '<pre>'.print_r($config, 1).'</pre>';
    $filename = $config['plugin_pathjs'].$req['sn'].'';
    if (!file_exists($filename)) {
      if ($this->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->File not found: '.print_r($filename, 1));
      echo '404';
      return FALSE;
    }

    $content = file_get_contents($filename);
    if (strpos($filename, 'executer.js') !== FALSE) {
      $repl    = array(
        'dt_method_mcpay_ccard'                => $config['ccardformid'],
        'dt_method_mcpay_sepa'                 => $config['sepaformid'],
        '"/micropayment/payment/getscript/sn/' => '"'.$config['plugin_urlshop'].'/mcpayGetScript?sn=',
      );
      $content = strtr($content, $repl);
    }
    if (strpos($filename, 'check.js') !== FALSE) {
      $repl    = array(
        'payformid'             => $config['payformid'],
        'mcpay_card_token-form' => $config['payformid'],
      );
      $content = strtr($content, $repl);
    }

    echo $content;
  }

  /**
   * setPaydata
   *
   * @param $paymethod
   * @param $paydata
   *
   * @return array|int|object
   */
  function setPaydata($paymethod, $paydata)
  {
    $userId = $_SESSION['Kunde']->kKunde;
    // set brand
    $cardType = 'visa';
    if (substr($paydata['CardPan'], 0, 1) == '5') $cardType = 'mastercard';
    $paydata['CardBrand'] = $cardType;

    if (!empty($paydata['TrxResult'])) {
      unset($paydata['TrxResult']);
    }

    // check if entry for paymethod and user already exists and update the old one
    $sql = 'SELECT * FROM xplugin_mcpay_mircopayment_paydata 
            WHERE `user_id` = "'.(int)$userId.'" 
            AND `paymethod` = "'.addslashes($paymethod).'"
            ';
    $res = Shop::DB()->query($sql, 2);
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$res', print_r($res, 1));

    if (!empty($res)) {
      $sql = 'UPDATE xplugin_mcpay_mircopayment_paydata 
              SET `data` = "'.addslashes(serialize($paydata)).'" 
              WHERE `user_id` = "'.(int)$userId.'" 
              AND `paymethod` = "'.addslashes($paymethod).'"
            ';
    } else {
      $sql = 'INSERT INTO xplugin_mcpay_mircopayment_paydata 
              SET `user_id` = "'.(int)$userId.'",
              `paymethod` = "'.addslashes($paymethod).'",
              `data` = "'.addslashes(serialize($paydata)).'" 
            ';
    }
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$sql', print_r($sql,1));
    return Shop::DB()->query($sql, 2);
  }

  /**
   * getPaydata
   *
   * @param $paymethod
   *
   * @return array|int|object
   */
  function getPaydata($paymethod)
  {
    $userId = $_SESSION['Kunde']->kKunde;
    $sql    = 'SELECT * FROM xplugin_mcpay_mircopayment_paydata 
            WHERE `user_id` = "'.(int)$userId.'"
            AND `paymethod` = "'.addslashes($paymethod).'" 
            ';
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$sql', print_r($sql,1));
    $paydata = Shop::DB()->query($sql, 9);
    return $paydata;
  }

}

?>