<?php
require_once('class.micropayment_base.php');

class micropayment_prepay extends micropayment_base
{
  var $pm        = 'prepay'; // important for event url
  var $paymethod = 'prepay_api';

  /**
   * preparePaymentProcess
   *
   * @param \Bestellung $order
   *
   * @return bool|void
   */
  function preparePaymentProcess($order)
  {
    global $smarty;

    $config = $this->getPluginConf($order);
    //echo '<pre>'.print_r($config,1).'</pre>';
    //echo '<pre>'.print_r($order,1).'</pre>';

    $userData = $this->getUserDataFromOrder($order, $config);
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$userData', print_r($userData,1));
    //echo '<pre>'.print_r($userData, 1).'</pre>';

    $hash = $this->generateHash($order);
    if ($order->cId != '') $hash = $order->cId;

    $notifyURL           = $this->getNotificationURL($hash).'&mcpaySessID=';
    $config['returnURL'] = $notifyURL;
    $key = ($this->duringCheckout) ? 'sh' : 'ph';
    $config['sessionID'] = $key.'-'.$hash;
    $config['payload']['cZahlungsID'] = str_replace('_', '', $hash);

    $return = new stdClass();

    try {
      $res = $this->McPay->bookPrepay($userData, $config);

      if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, $res); // log result
      $_SESSION['mcpay']['TrxResult'] = $res; // save result into session

      header('Location: '.$notifyURL); // redirect to thank you page over notify.php
      exit();

    } catch (Exception $e) {
      if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, $e->getMessage()); // log error
      $logo = mcpay::getPoweredBy(FALSE, $_SESSION['mcpay']['tracking']['chkSumData'], -1, $e->getMessage(), TRUE);
      $_SESSION['mcpay']['lastError'] = $e->getMessage().$logo;
      header('Location: bestellvorgang.php');
      die($e->getMessage());
    }

  }

  /**
   * handleNotification
   *
   * @param \Bestellung $order
   * @param string      $paymentHash
   * @param array       $args
   * @param bool        $returnURL
   *
   * @return string|void
   */
  function handleNotification($order, $paymentHash, $args, $returnURL = FALSE)
  {
    //$args = func_get_args();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$args', print_r($args, 1));
    if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, print_r($args, 1));

    $return = new stdClass();

    $crlf     = "\r\n";
    $errorMsg = '';
    $sessID   = '';
    if (!empty($args['mcpaySessID'])) {
      $sessID = $args['mcpaySessID'];
    }
    try {

      //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$_SESSION', print_r($_SESSION, 1));
      $config  = $this->getPluginConf($order);
      //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$config', print_r($config,1));
      $comment = $_SESSION['mcpay']['TrxResult']->prepayTextDE;
      if ($_SESSION['cISOSprache'] != 'ger') {
        $comment = $_SESSION['mcpay']['TrxResult']->prepayTextEN;
      }
      $this->setStatus(BESTELLUNG_STATUS_OFFEN, $order->cBestellNr, $comment);

      //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$this', print_r($this,1));
      if ($this->info->nWaehrendBestellung <= 0) {
        // send email with additional infos
        $object              = new stdClass();
        $object->tkunde      = $_SESSION['Kunde'];
        $object->tbestellung = $order;
        sendeMail(MAILTEMPLATE_BESTELLUNG_AKTUALISIERT, $object);
      }

    } catch (Exception $e) {
      if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, $e->getMessage()); // log error
      $errorMsg = $e->getMessage();
    }

    $url = $this->getReturnURL($order).'&mcpayerror='.$errorMsg;
    if (!empty($errorMsg)) {
      $url .= '&mcpayerror='.$errorMsg;
    }
    $return->forward    = 1;
    $return->forwardURL = $url;

    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$return', print_r($return,1));
    if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, print_r($return, 1));
    if ($returnURL) return $return->forwardURL;
    return $return;
  }

  /**
   * redirectOnPaymentSuccess
   *
   * @return bool
   */
  function redirectOnPaymentSuccess()
  {
    //$args = func_get_args();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$args', print_r($args,1));
    return TRUE;
  }

}
?>