<?php
require_once('class.micropayment_base.php');

class micropayment_sepa extends micropayment_base
{
  var $pm = 'sepa'; // important for event url
  var $paymethod = 'sepa_api';

  /**
   * handleAdditional
   *
   * @param array $req
   *
   * @return bool
   */
  function handleAdditional($req)
  {
    $_SESSION['mcpay_error'] = array();
    //echo '<pre>'.print_r($req, 1).'</pre>';
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$req', print_r($req,1));

    if (empty($_SESSION['Zahlungsart']->nWaehrendBestellung)){
      if (!empty($_SESSION['mcpay'])){
        $_SESSION['mcpay'] = array();
      }
      return true; // hide zusatzschritt if after order
    }
    // load texts
    $this->loadTexts();

    if (!empty($req['zahlungsartzusatzschritt'])) {
      if (!empty($req['mcpay_sepa_reuse']) && $req['mcpay_sepa_reuse'] != 'new') {
        // no inputs needed
        $_SESSION['mcpay']['SepaReuse'] = TRUE;
        $_SESSION['mcpay']['HOLDER'] = $req['mcpay_sepa_reuse_holder'];
        $_SESSION['mcpay']['BIC'] = $req['mcpay_sepa_reuse_bic'];
        $_SESSION['mcpay']['IBAN'] = $req['mcpay_sepa_reuse_iban'];
      } else {
        $_SESSION['mcpay']['SepaReuse'] = FALSE;

        // holder
        if (empty($req['mcpay_sepa_holder'])) {
          $_SESSION['mcpay_error'][] = $this->getText('mcpayAccountHolderError', 'Account Holder is mandatory.');
        } else {
          $_SESSION['mcpay']['HOLDER'] = $req['mcpay_sepa_holder'];
        }
        // iban
        if (empty($req['mcpay_sepa_iban'])) {
          $_SESSION['mcpay_error'][] = $this->getText('mcpayIBANError', 'IBAN is mandatory.');
        } else {
          $_SESSION['mcpay']['IBAN'] = $req['mcpay_sepa_iban'];
        }
        $ibanCountry = strtoupper(substr($_SESSION['mcpay']['IBAN'], 0, 2));
        if ($ibanCountry != 'DE') { // bic only mandatory when IBAN not from DE
          // bic
          if (empty($req['mcpay_sepa_bic'])) {
            $_SESSION['mcpay_error'][] = $this->getText('mcpayBICError', 'BIC is mandatory.');
          } else {
            $_SESSION['mcpay']['BIC'] = $req['mcpay_sepa_bic'];
          }
        } else {
          $_SESSION['mcpay']['BIC'] = $req['mcpay_sepa_bic'];
        }

        // save paymethod
        if (!empty($req['mcpay_sepa_remember'])) {
          // save before using account
          //$res = $this->setPaydata('sepa', $_SESSION['mcpay']);
          //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$res', print_r($res,1).$this->paymethod);
          $_SESSION['mcpay']['SepaRemember'] = TRUE;
        } else {
          $_SESSION['mcpay']['SepaRemember'] = FALSE;
        }
      }


      // error handling
      if (empty($_SESSION['mcpay_error'])) {
        return TRUE; // hide zusatzschritt
      } else {
        //echo '<pre>'.print_r($_SESSION['mcpay_error'], 1).'</pre>';
        $error_msg = '';
        foreach ($_SESSION['mcpay_error'] AS $k => $v) {
          $error_msg .= $v.'<br>';
        }

        global $smarty;
        $smarty->assign('error_msg', $error_msg);
      }
    }

    $this->preparePaymentProcess(FALSE);
    return FALSE; // show zusatzschritt
  }

  /**
   * preparePaymentProcess
   *
   * @param \Bestellung $order
   *
   * @return bool|void
   */
  function preparePaymentProcess($order)
  {
    global $smarty;

    $config = $this->getPluginConf($order);
    //echo '<pre>'.print_r($config,1).'</pre>';
    //echo '<pre>'.print_r($order,1).'</pre>';

    if (empty($_SESSION['Zahlungsart']->nWaehrendBestellung) && empty($_SESSION['mcpay'])){
      $order = false;
    }
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$_SESSION', print_r($_SESSION['mcpay'],1));

    if (!$order) { // form zusatzschritt
      if (!empty($_SESSION['mcpay'])) {
        unset($_SESSION['mcpay']);
      }
      $tpl = $this->getMyForm($config, $order);
      //echo '<pre>'.print_r($_SESSION['mcpay'], 1).'</pre>';

      $smarty->assign('micropayment_style', $config['plugin_urlshop'].'/mcpayGetStyle?sn=');
      $smarty->assign('micropayment_scripturl', $config['plugin_urlshop'].'/mcpayGetScript?sn=');
      $smarty->assign('micropayment_formid', $config['payformid']);
      $smarty->assign('micropayment_iframe', $tpl);
      $smarty->assign('micropayment_logo', $this->McPay->iconURLs['sepa']);

    } else { // book
      $userData = $this->getUserDataFromOrder($order, $config);
      //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$userData', print_r($userData,1));
      //echo '<pre>'.print_r($userData, 1).'</pre>';

      $hash = $this->generateHash($order);
      if ($order->cId != '') $hash = $order->cId;

      $notifyURL               = $this->getNotificationURL($hash).'&mcpaySessID=';
      $config['returnURL']     = $notifyURL;
      $config['returnURLNO3D'] = $notifyURL;
      $key = ($this->duringCheckout) ? 'sh' : 'ph';
      $config['sessionID'] = $key.'-'.$hash;
      $config['payload']['cZahlungsID'] = str_replace('_', '', $hash);

      $payData = array(
        'holder'   => $_SESSION['mcpay']['HOLDER'],
        'iban'     => $_SESSION['mcpay']['IBAN'],
        'bic'      => $_SESSION['mcpay']['BIC'],
        'remember' => $_SESSION['mcpay']['SepaRemember'],
      );

      $return          = new stdClass();
      $return->forward = TRUE;

      try {

        $res = $this->McPay->bookSepa($userData, $config, $payData);
        if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, $res); // log result
        $_SESSION['mcpay']['TrxResult'] = $res; // save result into session

        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$_SESSION', print_r($_SESSION['mcpay'],1));

        // save paymethod
        if (!empty($_SESSION['mcpay']['SepaRemember'])) {
          // save on success to be sure sepa account is valid and saved in mcp system
          $res = $this->setPaydata('sepa', $_SESSION['mcpay']);
          //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->Paydata saved', print_r($_SESSION['mcpay'],1).$this->paymethod);
        }

        $return->res = $res; // return result too
        header('Location: '.$notifyURL);
        exit();

      } catch (Exception $e) {
        if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, $e->getMessage()); // log error
        $logo = mcpay::getPoweredBy(FALSE, $_SESSION['mcpay']['tracking']['chkSumData'], -1, $e->getMessage(), TRUE);
        $_SESSION['mcpay']['lastError'] = $e->getMessage().$logo;
        header('Location: bestellvorgang.php');
        die($e->getMessage());
      }
    }
  }

  /**
   * getMyForm
   *
   * @param $conf
   *
   * @return string
   */
  function getMyForm($conf, $order)
  {
    $this->McPay->setConfig($conf);
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$conf', print_r($conf,1));
    $filename = $conf['plugin_pathview'].'/mcpay_form_sepa.tpl';

    $prefillData = array();
    $res         = $this->getPaydata('sepa');
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$res', print_r($res,1));
    if (!empty($res[0])) {
      foreach ($res AS $k => $v) {
        $payDat = unserialize($v['data']);
        //echo '<pre>'.print_r($payDat,1).'</pre>';
        $prefillData['use_'.$k] = array(
          'holder'  => $payDat['HOLDER'],
          'iban'    => $payDat['IBAN'],
          'bic'     => $payDat['BIC'],
          'checked' => 'checked',
        );
      }
    }

    // Template Translations
    $translation = array(
      'Name on Card'                => $this->getText('mcpayAccountHolder', 'Account Holder'),
      'Remember Data'               => $this->getText('mcpayRememberAccount', 'Remember Account for future purchase'),
      'Use new Card'                => $this->getText('mcpayUseNewAccount', 'Use new Account'),
      'Use old Card'                => $this->getText('mcpayUseOldAccount', 'Use old Account'),
      'Holder'                      => $this->getText('mcpayAccountHolder', 'Account Holder'),
      'IBAN'                        => $this->getText('mcpayIBAN', 'IBAN'),
      'BIC'                         => $this->getText('mcpayBIC', 'BIC'),
      //'Remove'                      => $this->getText('mcpay_', 'Remove'),
      'Please check your inputs!'   => $this->getText('mcpayCheckInputsError', 'Please check your inputs!'),
      'Vorname Nachname'            => $this->getText('mcpayFirstLastname', 'Firstname Lastname'),
      'NAME'                        => $this->getText('mcpayAccountHolder', 'Account Holder'),
    );

    $options = array(
      'showRemember' => TRUE,
      'showRemove'   => FALSE,
      'cardStyle'    => TRUE,
      'showCardOnly' => FALSE, // disable radio button for cards
      'hideImg'      => TRUE,  // hide logo
      'prefillPost'  => $_POST,
    );
    if (!empty($_POST['mcpay_sepa_reuse']) && $_POST['mcpay_sepa_reuse'] == 'new'){
      $options['preselectNewCard'] = TRUE;
    }

    $formTPL = $this->McPay->getSepaTPL($filename, $options, $prefillData, $translation);
    $output = $formTPL;

    $output.= '<script>';
    if ($this->McPay->debug){
      $output.= 'console.log("MCPAY ModelSepa START");';
    }

    $output.= 'mcpay_sepa_execute();';

    if ($this->McPay->debug){
      $output.= 'console.log("MCPAY ModelSepa END");';
    }

    $output.= '</script>';
    return $output;
  }

  /**
   * handleNotification
   *
   * @param \Bestellung $order
   * @param string      $paymentHash
   * @param array       $args
   * @param bool        $returnURL
   *
   * @return string|void
   */
  function handleNotification($order, $paymentHash, $args, $returnURL = FALSE)
  {
    //$args = func_get_args();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$args', print_r($args, 1));

    $crlf     = "\r\n";
    $return = new stdClass();
    $errorMsg = '';
    try {

      //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$_SESSION', print_r($_SESSION['mcpay'], 1));
      $config  = $this->getPluginConf($order);
      $comment = $_SESSION['mcpay']['TrxResult']->mandateTextDE;
      if ($_SESSION['cISOSprache'] != 'ger') {
        $comment = $_SESSION['mcpay']['TrxResult']->mandateTextEN;
      }
      $this->setStatus(BESTELLUNG_STATUS_OFFEN, $order->cBestellNr, $comment);

      //$return = parent::handleNotification($order, $paymentHash, $args, true);

      $url = $this->getReturnURL($order);

      $incomingPayment           = new stdClass();
      $incomingPayment->fBetrag  = $order->fGesamtsummeKundenwaehrung;
      $incomingPayment->cISO     = $order->Waehrung->cISO;
      $incomingPayment->cHinweis = $_SESSION['mcpay']['TrxResult']->resCreate->payToken;

      $this->addIncomingPayment($order, $incomingPayment);
      $this->setOrderStatusToPaid($order);
      $this->sendConfirmationMail($order);
      $this->updateNotificationID($order->kBestellung, $paymentHash);

      header('Location: '.$url);
      exit();

    } catch (Exception $e) {
      if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, $e->getMessage()); // log error
      $errorMsg = $e->getMessage();
    }

    if (!empty($errorMsg)){
      $url = $this->getReturnURL($order).'&mcpayerror='.$errorMsg;
      $return->forward = 1;
      $return->forwardURL = $url;
    }

    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$return', print_r($return,1));
    if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, print_r($return,1));
    if ($returnURL) return $return->forwardURL;
    echo $return;
  }

}

?>